@@ -1,14 +1,14 @@
 # Copyright 2008,2009 Marcus Huewe <suse-tux@gmx.de>
-# 
+#
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License version 2
 # as published by the Free Software Foundation;
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
@@ -97,7 +97,7 @@ class SectionLine(Line):
         Line.__init__(self, sectname, 'section')
         self._lines = []
         self._dict = dict
-    
+
     def _find(self, name):
         for line in self._lines:
             if line.name == name:
@@ -105,12 +105,12 @@ class SectionLine(Line):
         return None
 
     def _add_option(self, optname, value = None, line = None, sep = '='):
-        if not (value or line):
-            raise Error('Either value or line must be passed in')
+        if value is None and line is None:
+            raise ConfigParser.Error('Either value or line must be passed in')
         elif value and line:
-            raise Error('value and line are mutually exclusive')
+            raise ConfigParser.Error('value and line are mutually exclusive')
 
-        if value:
+        if value is not None:
             line = '%s%s%s' % (optname, sep, value)
         opt = self._find(optname)
         if opt:
