@@ -1,12 +1,17 @@
-# Copyright (C) 2006 Novell Inc.  All rights reserved.
+# Copyright (C) 2006-2009 Novell Inc.  All rights reserved.
 # This program is free software; it may be used, copied, modified
 # and distributed under the terms of the GNU General Public Licence,
-# either version 2, or (at your option) any later version.
+# either version 2, or version 3 (at your option).
 
 """Read osc configuration and store it in a dictionary
 
 This module reads and parses ~/.oscrc. The resulting configuration is stored
-for later usage in a dictionary named 'config'. 
+for later usage in a dictionary named 'config'.
+The .oscrc is kept mode 0600, so that it is not publically readable.
+This gives no real security for storing passwords.
+If in doubt, use your favourite keyring.
+Password is stored on ~/.oscrc as bz2 compressed and base64 encoded, so that is fairly
+large and not to be recognized or remembered easily by an occasional spectator.
 
 If information is missing, it asks the user questions.
 
@@ -31,47 +36,93 @@ The configuration dictionary could look like this:
 
 import OscConfigParser
 from osc import oscerr
+from oscsslexcp import NoSecureSSLError
+import os
+
+GENERIC_KEYRING = False
+GNOME_KEYRING = False
 
 try:
-    import gobject
-    gobject.set_application_name('osc')
-    import gnomekeyring
-    GNOME_KEYRING = gnomekeyring.is_available()
-except:
-    GNOME_KEYRING = False
+    import keyring
+    GENERIC_KEYRING = True
 
-# being global to this module, this dict can be accessed from outside
-# it will hold the parsed configuration
-config = { }
+except:
+    try:
+        import gobject
+        gobject.set_application_name('osc')
+        import gnomekeyring
+        if os.environ['GNOME_DESKTOP_SESSION_ID']:
+            # otherwise gnome keyring bindings spit out errors, when you have
+            # it installed, but you are not under gnome
+            # (even though hundreds of gnome-keyring daemons got started in parallel)
+            # another option would be to support kwallet here
+            GNOME_KEYRING = gnomekeyring.is_available()
+    except:
+        pass
 
 DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              'user': 'your_username',
              'pass': 'your_password',
+             'passx': '',
              'packagecachedir': '/var/tmp/osbuild-packagecache',
              'su-wrapper': 'su -c',
 
              # build type settings
              'build-cmd': '/usr/bin/build',
-             'build-type' : '', # may be empty for chroot, kvm or xen
+             'build-type': '', # may be empty for chroot, kvm or xen
              'build-root': '/var/tmp/build-root',
              'build-device': '', # required for VM builds
-             'build-memory' : '',# required for VM builds
-             'build-swap' : '',  # optional for VM builds
+             'build-memory': '',# required for VM builds
+             'build-swap': '',  # optional for VM builds
+             'build-jobs': os.sysconf('SC_NPROCESSORS_ONLN'), # compile with N jobs
 
              'debug': '0',
              'http_debug': '0',
              'traceback': '0',
              'post_mortem': '0',
+             'use_keyring': '1',
              'gnome_keyring': '1',
              'cookiejar': '~/.osc_cookiejar',
              # enable project tracking by default
              'do_package_tracking': '1',
              # default for osc build
-             'extra-pkgs': 'vim gdb strace',
-             # default platform
-             'build_platform': 'openSUSE_Factory',
+             'extra-pkgs': '',
+             # default repository
+             'build_repository': 'openSUSE_Factory',
+             # default project for branch or bco
+             'getpac_default_project': 'openSUSE:Factory',
+             # alternate filesystem layout: have multiple subdirs, where colons were.
+             'checkout_no_colon': '0',
+             # local files to ignore with status, addremove, ....
+             # local files to ignore with status, addremove, ....
+             'exclude_glob': '.osc CVS .svn .* _linkerror *~ #*# *.orig *.bak',
+             # keep passwords in plaintext. If you see this comment, your osc
+             # already uses the encrypted password, and only keeps them in plain text
+             # for backwards compatibility. Default will change to 0 in future releases.
+             'plaintext_passwd': '1',
+             # limit the age of requests shown with 'osc req list'.
+             # this is a default only, can be overridden by 'osc req list -D NNN'
+             # Use 0 for unlimted.
+             'request_list_days': 30,
+             # check for unversioned/removed files before commit
+             'check_filelist': '1',
+             # check for pending requests after executing an action (e.g. checkout, update, commit)
+             'check_for_request_on_action': '0',
+             # what to do with the source package if the submitrequest has been accepted
+             'submitrequest_on_accept_action': '',
+             'linkcontrol': '0',
+
+             # Maintenance defaults to OBS instance defaults
+             'maintained_attribute': 'OBS:Maintained',
+             'maintained_update_project_attribute': 'OBS:UpdateProject',
 }
-boolean_opts = ['debug', 'do_package_tracking', 'http_debug', 'post_mortem', 'traceback']
+
+# being global to this module, this dict can be accessed from outside
+# it will hold the parsed configuration
+config = DEFAULTS.copy()
+
+boolean_opts = ['debug', 'do_package_tracking', 'http_debug', 'post_mortem', 'traceback', 'check_filelist', 'plaintext_passwd',
+    'checkout_no_colon', 'check_for_request_on_action', 'linkcontrol']
 
 new_conf_template = """
 [general]
@@ -92,6 +143,9 @@ apiurl = %(apiurl)s
 # /srv/oscbuild/%%(repo)s-%%(arch)s-%%(project)s-%%(package)s
 #build-root = %(build-root)s
 
+# compile with N jobs (default: "getconf _NPROCESSORS_ONLN")
+#build-jobs = N
+
 # extra packages to install when building packages locally (osc build)
 # this corresponds to osc build's -x option and can be overridden with that
 # -x '' can also be given on the command line to override this setting, or
@@ -99,30 +153,62 @@ apiurl = %(apiurl)s
 #extra-pkgs = vim gdb strace
 
 # build platform is used if the platform argument is omitted to osc build
-#build_platform = openSUSE_Factory
+#build_repository = %(build_repository)s
 
-# show info useful for debugging 
+# default project for getpac or bco
+#getpac_default_project = %(getpac_default_project)s
+
+# alternate filesystem layout: have multiple subdirs, where colons were.
+#checkout_no_colon = %(checkout_no_colon)s
+
+# local files to ignore with status, addremove, ....
+#exclude_glob = %(exclude_glob)s
+
+# keep passwords in plaintext. If you see this comment, your osc
+# already uses the encrypted password, and only keeps them in plain text
+# for backwards compatibility. Default will change to 0 in future releases.
+# You can remove the plaintext password without harm, if you do not need
+# backwards compatibility.
+#plaintext_passwd = %(plaintext_passwd)s
+
+# limit the age of requests shown with 'osc req list'.
+# this is a default only, can be overridden by 'osc req list -D NNN'
+# Use 0 for unlimted.
+#request_list_days = %(request_list_days)s
+
+# show info useful for debugging
 #debug = 1
-    
-# show HTTP traffic useful for debugging 
+
+# show HTTP traffic useful for debugging
 #http_debug = 1
-    
+
 # jump into the debugger in case of errors
 #post_mortem = 1
-    
+
 # print call traces in case of errors
 #traceback = 1
-    
-# use GNOME keyring for credentials if available
-#gnome_keyring = 0
-    
+
+# use KDE/Gnome/MacOS/Windows keyring for credentials if available
+#use_keyring = 1
+
+# check for unversioned/removed files before commit
+#check_filelist = 1
+
+# check for pending requests after executing an action (e.g. checkout, update, commit)
+#check_for_request_on_action = 0
+
+# what to do with the source package if the submitrequest has been accepted. If
+# nothing is specified the API default is used
+#submitrequest_on_accept_action = cleanup|update|noupdate
+
 [%(apiurl)s]
 user = %(user)s
 pass = %(pass)s
+passx = %(passx)s
 # set aliases for this apiurl
 # aliases = foo, bar
 # email used in .changes, unless the one from osc meta prj <user> will be used
-# email = 
+# email =
 # additional headers to pass to a request, e.g. for special authentication
 #http_headers = Host: foofoobar,
 #       User: mumblegack
@@ -157,7 +243,8 @@ def parse_apisrv_url(scheme, apisrv):
     if apisrv.startswith('http://') or apisrv.startswith('https://'):
         return urlparse.urlsplit(apisrv)[0:2]
     elif scheme != None:
-        return scheme, apisrv
+        # the split/join is needed to get a proper url (e.g. without a trailing slash)
+        return urlparse.urlsplit(urljoin(scheme, apisrv))[0:2]
     else:
         from urllib2 import URLError
         msg = 'invalid apiurl \'%s\' (specify the protocol (http:// or https://))' % apisrv
@@ -200,23 +287,37 @@ def get_apiurl_usr(apiurl):
             % (apiurl, config['user'])
         return config['user']
 
+
+
 def init_basicauth(config):
     """initialize urllib2 with the credentials for Basic Authentication"""
 
     from osc.core import __version__
-    import os, urllib2
     import cookielib
 
+    if config['api_host_options'][config['apiurl']]['sslcertck']:
+        try:
+            import oscssl
+            oscssl.myHTTPSConnection.appname = 'osc'
+            oscssl.myProxyHTTPSConnection.appname = 'osc'
+            from M2Crypto import m2urllib2
+        except Exception, e:
+            print e
+            raise NoSecureSSLError("M2Crypto is needed to access %s in a secure way.\nPlease install python-m2crypto." % config['apiurl'])
+
+    import urllib2
+
+
     global cookiejar
 
-    # HTTPS proxy is not supported by urllib2. It only leads to an error
-    # or, at best, a warning.
-    # https://bugzilla.novell.com/show_bug.cgi?id=214983
-    # https://bugzilla.novell.com/show_bug.cgi?id=298378
-    if 'https_proxy' in os.environ:
-        del os.environ['https_proxy']
-    if 'HTTPS_PROXY' in os.environ:
-        del os.environ['HTTPS_PROXY']
+    import sys
+    if sys.version_info < (2, 6):
+       # HTTPS proxy is not supported in old urllib2. It only leads to an error
+       # or, at best, a warning.
+       if 'https_proxy' in os.environ:
+           del os.environ['https_proxy']
+       if 'HTTPS_PROXY' in os.environ:
+           del os.environ['HTTPS_PROXY']
 
     if config['http_debug']:
         # brute force
@@ -229,7 +330,7 @@ def init_basicauth(config):
 
     cookie_file = os.path.expanduser(config['cookiejar'])
     cookiejar = cookielib.LWPCookieJar(cookie_file)
-    try: 
+    try:
         cookiejar.load(ignore_discard=True)
     except IOError:
         try:
@@ -239,7 +340,29 @@ def init_basicauth(config):
             #print 'Unable to create cookiejar file: \'%s\'. Using RAM-based cookies.' % cookie_file
             cookiejar = cookielib.CookieJar()
 
-    opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cookiejar), authhandler)
+
+    if config['api_host_options'][config['apiurl']]['sslcertck']:
+        cafile = capath = None
+        if 'capath' in config['api_host_options'][config['apiurl']]:
+            capath = config['api_host_options'][config['apiurl']]['capath']
+        if 'cafile' in config['api_host_options'][config['apiurl']]:
+            cafile = config['api_host_options'][config['apiurl']]['cafile']
+        if not cafile and not capath:
+            for i in ['/etc/pki/tls/cert.pem', '/etc/ssl/certs' ]:
+                if os.path.isfile(i):
+                    cafile = i
+                    break
+                elif os.path.isdir(i):
+                    capath = i
+                    break
+        ctx = oscssl.mySSLContext()
+        if ctx.load_verify_locations(capath=capath, cafile=cafile) != 1: raise Exception('No CA certificates found')
+        opener = m2urllib2.build_opener(ctx, oscssl.myHTTPSHandler(ctx), urllib2.HTTPCookieProcessor(cookiejar), authhandler)
+    else:
+        import sys
+        print >>sys.stderr, "WARNING: SSL certificate checks disabled. Connection is insecure!\n"
+        opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cookiejar), authhandler)
+
     urllib2.install_opener(opener)
 
     opener.addheaders = [('User-agent', 'osc/%s' % __version__)]
@@ -256,7 +379,6 @@ def get_configParser(conffile=None, force_read=False):
     ConfigParser object is stored in a method attribute and this attribute
     is returned unless you pass force_read=True.
     """
-    import os
     conffile = conffile or os.environ.get('OSC_CONFIG', '~/.oscrc')
     conffile = os.path.expanduser(conffile)
     if force_read or not get_configParser.__dict__.has_key('cp'):
@@ -271,11 +393,18 @@ def write_initial_config(conffile, entries, custom_template = ''):
     for the config file (e.g. { 'user' : 'username', 'pass' : 'password' } ).
     custom_template is an optional configuration template.
     """
-    import os, StringIO, sys
+    import StringIO, sys
     conf_template = custom_template or new_conf_template
     config = DEFAULTS.copy()
     config.update(entries)
-    if config['gnome_keyring'] and GNOME_KEYRING:
+    config['passx'] = config['pass'].encode('bz2').encode('base64')
+    if config['use_keyring'] and GENERIC_KEYRING:
+        protocol, host = \
+            parse_apisrv_url(None, config['apiurl'])
+        keyring.set_password(host, config['user'], config['pass'])
+        config['pass'] = ''
+        config['passx'] = ''
+    elif config['gnome_keyring'] and GNOME_KEYRING:
         protocol, host = \
             parse_apisrv_url(None, config['apiurl'])
         gnomekeyring.set_network_password_sync(
@@ -285,6 +414,9 @@ def write_initial_config(conffile, entries, custom_template = ''):
             server = host)
         config['user'] = ''
         config['pass'] = ''
+        config['passx'] = ''
+    if not config['plaintext_passwd']:
+        config['pass'] = ''
     sio = StringIO.StringIO(conf_template.strip() % config)
     cp = OscConfigParser.OscConfigParser(DEFAULTS)
     cp.readfp(sio)
@@ -314,7 +446,15 @@ def add_section(filename, url, user, passwd):
     except OscConfigParser.ConfigParser.DuplicateSectionError:
         # Section might have existed, but was empty
         pass
-    if config['gnome_keyring'] and GNOME_KEYRING:
+    if config['use_keyring'] and GENERIC_KEYRING:
+        protocol, host = \
+            parse_apisrv_url(None, url)
+        keyring.set_password(host, user, passwd)
+        cp.set(url, 'keyring', '1')
+        cp.set(url, 'user', user)
+        cp.remove_option(url, 'pass')
+        cp.remove_option(url, 'passx')
+    elif config['gnome_keyring'] and GNOME_KEYRING:
         protocol, host = \
             parse_apisrv_url(None, url)
         gnomekeyring.set_network_password_sync(
@@ -323,23 +463,28 @@ def add_section(filename, url, user, passwd):
             protocol = protocol,
             server = host)
         cp.set(url, 'keyring', '1')
+        cp.remove_option(url, 'pass')
+        cp.remove_option(url, 'passx')
     else:
         cp.set(url, 'user', user)
-        cp.set(url, 'pass', passwd)
+        if not config['plaintext_passwd']:
+            cp.remove_option(url, 'pass')
+        cp.set(url, 'passx', passwd.encode('bz2').encode('base64'))
     file = open(filename, 'w')
     cp.write(file, True)
     if file: file.close()
 
 
-def get_config(override_conffile = None, 
+def get_config(override_conffile = None,
                override_apiurl = None,
-               override_debug = None, 
-               override_http_debug = None, 
+               override_debug = None,
+               override_http_debug = None,
                override_traceback = None,
                override_post_mortem = None,
-               override_no_gnome_keyring = None):
+               override_no_keyring = None,
+               override_no_gnome_keyring = None,
+               override_verbose = None):
     """do the actual work (see module documentation)"""
-    import os
     import sys
     import re
     global config
@@ -353,6 +498,9 @@ def get_config(override_conffile = None,
 
     # okay, we made sure that .oscrc exists
 
+    # make sure it is not world readable, it may contain a password.
+    os.chmod(conffile, 0600)
+
     cp = get_configParser(conffile)
 
     if not cp.has_section('general'):
@@ -363,18 +511,6 @@ def get_config(override_conffile = None,
 
     config = dict(cp.items('general', raw=1))
 
-    # backward compatibility
-    if config.has_key('apisrv'):
-        apisrv = config['apisrv'].lstrip('http://')
-        apisrv = apisrv.lstrip('https://')
-        scheme = config.get('scheme', 'https')
-        config['apiurl'] = urljoin(scheme, apisrv)
-    if config.has_key('apisrv') or config.has_key('scheme'):
-        import warnings
-        warnings.warn('Use of the \'scheme\' or \'apisrv\' config option is deprecated! ' \
-                      'See README for migration details.',
-                      DeprecationWarning)
-
     for i in boolean_opts:
         try:
             config[i] = cp.getboolean('general', i)
@@ -382,11 +518,10 @@ def get_config(override_conffile = None,
             raise oscerr.ConfigError('cannot parse \'%s\' setting: ' % i + str(e), conffile)
 
     config['packagecachedir'] = os.path.expanduser(config['packagecachedir'])
+    config['exclude_glob'] = config['exclude_glob'].split()
 
     re_clist = re.compile('[, ]+')
     config['extra-pkgs'] = [ i.strip() for i in re_clist.split(config['extra-pkgs'].strip()) if i ]
-    if config['extra-pkgs'] == []: 
-        config['extra-pkgs'] = None
 
     # collect the usernames, passwords and additional options for each api host
     api_host_options = {}
@@ -400,7 +535,9 @@ def get_config(override_conffile = None,
 
     # override values which we were called with
     # This needs to be done before processing API sections as it might be already used there
-    if override_no_gnome_keyring: 
+    if override_no_keyring:
+        config['use_keyring'] = False
+    if override_no_gnome_keyring:
         config['gnome_keyring'] = False
 
     aliases = {}
@@ -410,8 +547,16 @@ def get_config(override_conffile = None,
             parse_apisrv_url(config.get('scheme', 'https'), url)
         apiurl = urljoin(scheme, host)
         user = None
-        # Read from gnome keyring if available
-        if config['gnome_keyring'] and GNOME_KEYRING:
+        if config['use_keyring'] and GENERIC_KEYRING:
+            try:
+                # Read from keyring lib if available
+                user = cp.get(url, 'user', raw=True)
+                password = keyring.get_password(host, user)
+            except:
+                # Fallback to file based auth.
+                pass
+        elif config['gnome_keyring'] and GNOME_KEYRING:
+            # Read from gnome keyring if available
             try:
                 gk_data = gnomekeyring.find_network_password_sync(
                     protocol = scheme,
@@ -427,12 +572,17 @@ def get_config(override_conffile = None,
             #from the general section.
             user         = cp.get(url, 'user', raw=True) # need to set raw to prevent '%' expansion
             password     = cp.get(url, 'pass', raw=True) # especially on password!
-            if cp.has_option(url, 'keyring') and cp.get(url, 'keyring'):
-                # This APIURL was configured to use keyring by
-                continue
-        email        = ''
-        if cp.has_option(url, 'email'):
-            email    = cp.get(url, 'email')
+            passwordx    = cp.get(url, 'passx', raw=True) # especially on password!
+            if password is None or password == 'your_password':
+                try:
+                    password = passwordx.decode('base64').decode('bz2')
+                except:
+                    print "%s: no credentials known" % url
+                    password = 'your_password'
+            else:
+                if not passwordx:
+                    print "%s: rewriting from plain pass to encoded pass\n" % url
+                    add_section(conffile, url, user, password)
 
         if cp.has_option(url, 'http_headers'):
             http_headers = cp.get(url, 'http_headers')
@@ -452,14 +602,43 @@ def get_config(override_conffile = None,
         api_host_options[apiurl] = { 'user': user,
                                      'pass': password,
                                      'http_headers': http_headers}
-        if email:
-            api_host_options[apiurl]['email'] = email
+
+        optional = ('email', 'sslcertck', 'cafile', 'capath')
+        for key in optional:
+            if cp.has_option(url, key):
+                if key == 'sslcertck':
+                    api_host_options[apiurl][key] = cp.getboolean(url, key)
+                else:
+                    api_host_options[apiurl][key] = cp.get(url, key)
+
+        if not 'sslcertck' in api_host_options[apiurl]:
+            api_host_options[apiurl]['sslcertck'] = True
 
     # add the auth data we collected to the config dict
     config['api_host_options'] = api_host_options
+    config['apiurl_aliases'] = aliases
+
+    apiurl = aliases.get(config['apiurl'], config['apiurl'])
+    config['apiurl'] = urljoin(*parse_apisrv_url(None, apiurl))
+    # backward compatibility
+    if config.has_key('apisrv'):
+        apisrv = config['apisrv'].lstrip('http://')
+        apisrv = apisrv.lstrip('https://')
+        scheme = config.get('scheme', 'https')
+        config['apiurl'] = urljoin(scheme, apisrv)
+    if config.has_key('apisrv') or config.has_key('scheme'):
+        print >>sys.stderr, 'Warning: Use of the \'scheme\' or \'apisrv\' in ~/.oscrc is deprecated!\n' \
+                            'Warning: See README for migration details.'
+    if config.has_key('build_platform') and not config.has_key('build_repository'):
+        config['build_repository'] = config['build_platform']
 
     # override values which we were called with
-    if override_debug: 
+    if override_verbose:
+        config['verbose'] = override_verbose + 1
+    if not config.has_key('verbose'):
+        config['verbose'] = 1
+
+    if override_debug:
         config['debug'] = override_debug
     if override_http_debug:
         config['http_debug'] = override_http_debug
@@ -470,10 +649,9 @@ def get_config(override_conffile = None,
     if override_apiurl:
         apiurl = aliases.get(override_apiurl, override_apiurl)
         # check if apiurl is a valid url
-        parse_apisrv_url(None, apiurl)
-        config['apiurl'] = apiurl
+        config['apiurl'] = urljoin(*parse_apisrv_url(None, apiurl))
 
-    # XXX unless config['user'] goes away (and is replaced with a handy function, or 
+    # XXX unless config['user'] goes away (and is replaced with a handy function, or
     # config becomes an object, even better), set the global 'user' here as well,
     # provided that there _are_ credentials for the chosen apiurl:
     try:
@@ -486,3 +664,4 @@ def get_config(override_conffile = None,
     # finally, initialize urllib2 for to use the credentials for Basic Authentication
     init_basicauth(config)
 
+# vim: sw=4 et
