@@ -7,7 +7,7 @@ import sys, os
 import urllib2
 from urlgrabber.grabber import URLGrabber, URLGrabError
 from urlgrabber.mirror import MirrorGroup
-from core import data_from_rpm
+from util import packagequery
 try:
     from meter import TextMeter
 except:
@@ -17,12 +17,12 @@ except:
 def join_url(self, base_url, rel_url):
     """to override _join_url of MirrorGroup, because we want to
     pass full URLs instead of base URL where relative_url is added later...
-    IOW, we make MirrorGroup ignore relative_url""" 
+    IOW, we make MirrorGroup ignore relative_url"""
     return base_url
 
 
 class Fetcher:
-    def __init__(self, cachedir = '/tmp', api_host_options = {}, urllist = [], http_debug = False, cookiejar = None):
+    def __init__(self, cachedir = '/tmp', api_host_options = {}, urllist = [], http_debug = False, cookiejar = None, offline = False):
 
         __version__ = '0.1'
         __user_agent__ = 'osbuild/%s' % __version__
@@ -37,6 +37,7 @@ class Fetcher:
         self.cachedir = cachedir
         self.urllist = urllist
         self.http_debug = http_debug
+        self.offline = offline
 
         passmgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
         for host in api_host_options.keys():
@@ -62,13 +63,15 @@ class Fetcher:
         raise errobj.exception
 
 
-    def fetch(self, pac):
+    def fetch(self, pac, prefix=''):
         # for use by the failure callback
         self.curpac = pac
 
+        if self.offline:
+            return True
+
         MirrorGroup._join_url = join_url
         mg = MirrorGroup(self.gr, pac.urllist)
-        partname = pac.fullfilename + ".part"
 
         if self.http_debug:
             print
@@ -78,33 +81,32 @@ class Fetcher:
 
         try:
             # it returns the filename
-            ret = mg.urlgrab(pac.filename, 
-                             filename=partname,
-                             text = '(%s) %s' %(pac.project, pac.filename))
-
+            ret = mg.urlgrab(pac.filename,
+                             filename = pac.fullpartname,
+                             text = '%s(%s) %s' %(prefix, pac.project, pac.filename))
         except URLGrabError, e:
-            print
             print >>sys.stderr, 'Error:', e.strerror
             print >>sys.stderr, 'Failed to retrieve %s from the following locations (in order):' % pac.filename
             print >>sys.stderr, '\n'.join(pac.urllist)
-
             sys.exit(1)
 
-        if partname.endswith('.rpm.part'):
-            rpm_data = data_from_rpm(partname, 'Name:', 'Version:', 'Release:', 'Arch:', 'SourceRPM:', 'NoSource:', 'NoPatch:')
-            if rpm_data:
-                arch = rpm_data['Arch:']
-                if not rpm_data['SourceRPM:']:
-                    if rpm_data['NoSource:'] or rpm_data['NoPatch:']:
-                        arch = "nosrc"
-                    else:
-                        arch = "src"
-                canonname = "%s-%s-%s.%s.rpm" % (rpm_data['Name:'], rpm_data['Version:'], rpm_data['Release:'], arch)
-                head, tail = os.path.split(pac.fullfilename)
-                pac.filename = canonname
-                pac.fullfilename = os.path.join(head, canonname)
+        pkgq = packagequery.PackageQuery.query(pac.fullpartname, extra_rpmtags=(1044, 1051, 1052))
+        arch = pkgq.arch()
+        # SOURCERPM = 1044
+        if pkgq.filename_suffix == 'rpm' and not pkgq.getTag(1044):
+            # NOSOURCE = 1051, NOPATCH = 1052
+            if pkgq.getTag(1051) or pkgq.getTag(1052):
+                arch = "nosrc"
+            else:
+                arch = "src"
+        if pkgq.release():
+            canonname = '%s-%s-%s.%s.%s' % (pkgq.name(), pkgq.version(), pkgq.release(), arch, pkgq.filename_suffix)
+        else:
+            canonname = '%s-%s.%s.%s' % (pkgq.name(), pkgq.version(), arch, pkgq.filename_suffix)
+        pac.filename = canonname
+        pac.fullfilename = os.path.join(pac.localdir, canonname)
 
-        os.rename(partname, pac.fullfilename);
+        os.rename(pac.fullpartname, pac.fullfilename)
 
     def dirSetup(self, pac):
         dir = os.path.join(self.cachedir, pac.localdir)
@@ -118,33 +120,44 @@ class Fetcher:
 
 
     def run(self, buildinfo):
+        cached = 0
+        all = len(buildinfo.deps)
         for i in buildinfo.deps:
             i.makeurls(self.cachedir, self.urllist)
-
-            if os.path.exists(os.path.join(i.localdir, i.fullfilename)):
-                #print 'cached:', i.fullfilename
-                pass
-            else:
+            if os.path.exists(i.fullfilename):
+                cached += 1
+        miss = 0
+        needed = all - cached
+        if all:
+            miss = 100.0 * needed / all
+        print "%.1f%% cache miss. %d/%d dependencies cached.\n" % (miss, cached, all)
+        done = 1
+        for i in buildinfo.deps:
+            i.makeurls(self.cachedir, self.urllist)
+            if not os.path.exists(i.fullfilename):
                 self.dirSetup(i)
-
                 try:
                     # if there isn't a progress bar, there is no output at all
                     if not self.progress_obj:
-                        print '(%s) %s' % (i.project, i.filename)
+                        print '%d/%d (%s) %s' % (done, needed, i.project, i.filename)
                     self.fetch(i)
+                    if self.progress_obj:
+                        print "  %d/%d\r" % (done,needed),
+                        sys.stdout.flush()
 
                 except KeyboardInterrupt:
                     print 'Cancelled by user (ctrl-c)'
                     print 'Exiting.'
-                    if os.path.exists(i.fullfilename):
-                        print 'Cleaning up incomplete file', i.fullfilename
-                        os.unlink(i.fullfilename)
+                    if os.path.exists(i.fullpartname):
+                        print 'Cleaning up incomplete file', i.fullpartname
+                        os.unlink(i.fullpartname)
                     sys.exit(0)
+            done += 1
 
 
 
 def verify_pacs(pac_list):
-    """Take a list of rpm filenames and run rpm -K on them. 
+    """Take a list of rpm filenames and run rpm -K on them.
 
        In case of failure, exit.
 
@@ -155,7 +168,7 @@ def verify_pacs(pac_list):
 
     if not pac_list:
         return
-        
+
     # don't care about the return value because we check the
     # output anyway, and rpm always writes to stdout.
 
@@ -167,19 +180,19 @@ def verify_pacs(pac_list):
                 stderr=subprocess.STDOUT, close_fds=True).stdout
 
     # restore locale
-    if saved_LC_ALL: os.environ['LC_ALL'] = saved_LC_ALL;
+    if saved_LC_ALL: os.environ['LC_ALL'] = saved_LC_ALL
     else: os.environ.pop('LC_ALL')
 
     for line in o.readlines():
 
         if not 'OK' in line:
-            print 
+            print
             print >>sys.stderr, 'The following package could not be verified:'
             print >>sys.stderr, line
             sys.exit(1)
 
         if 'NOT OK' in line:
-            print 
+            print
             print >>sys.stderr, 'The following package could not be verified:'
             print >>sys.stderr, line
 
@@ -199,7 +212,7 @@ def verify_pacs(pac_list):
 - If you do not trust the packages, you should configure osc build for XEN or KVM
 
 - You may use --no-verify to skip the verification (which is a risk for your system).
-""" % {'name': missing_key, 
+""" % {'name': missing_key,
        'dir': os.path.expanduser('~')}
 
             else:
