@@ -1,14 +1,14 @@
 # Copyright 2009 Marcus Huewe <suse-tux@gmx.de>
-# 
+#
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License version 2
 # as published by the Free Software Foundation;
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
@@ -19,6 +19,10 @@ import stat
 import struct
 import sys
 
+# workaround for python24
+if not hasattr(os, 'SEEK_SET'):
+    os.SEEK_SET = 0
+
 # format implementation is based on src/copyin.c and src/util.c (see cpio sources)
 
 class CpioError(Exception):
@@ -65,7 +69,7 @@ class CpioHdr:
     def __str__(self):
         return "%s %s %s %s" % (self.filename, self.filesize, self.namesize, self.dataoff)
 
-class Cpio:
+class CpioRead:
     """
     Represents a cpio archive.
     Supported formats:
@@ -141,9 +145,9 @@ class Cpio:
             self.__file = open(self.filename, 'rb')
             try:
                 if sys.platform[:3] != 'win':
-                    self.__file = mmap.mmap(self.__file.fileno(), 0, prot = mmap.PROT_READ)
+                    self.__file = mmap.mmap(self.__file.fileno(), os.path.getsize(self.__file.name), prot = mmap.PROT_READ)
                 else:
-                    self.__file = mmap.mmap(self.__file.fileno(), 0)
+                    self.__file = mmap.mmap(self.__file.fileno(), os.path.getsize(self.__file.name))
             except EnvironmentError, e:
                 if e.errno == 19 or ( hasattr(e, 'winerror') and e.winerror == 5 ):
                     print >>sys.stderr, 'cannot use mmap to read the file, failing back to default'
@@ -197,3 +201,52 @@ class Cpio:
         dest = dest or os.getcwd()
         for h in self.hdrs:
             self._copyin_file(h, dest, h.filename)
+
+class CpioWrite:
+    """cpio archive small files in memory, using new style portable header format"""
+
+    def __init__(self):
+        self.cpio = ''
+
+    def add(self, name=None, content=None, perms=0x1a4, type=0x8000):
+        namesize = len(name) + 1
+        if namesize % 2:
+            name += '\0'
+        filesize = len(content)
+        mode = perms | type
+
+        c = []
+        c.append('070701') # magic
+        c.append('%08X' % 0) # inode
+        c.append('%08X' % mode) # mode
+        c.append('%08X' % 0) # uid
+        c.append('%08X' % 0) # gid
+        c.append('%08X' % 0) # nlink
+        c.append('%08X' % 0) # mtime
+        c.append('%08X' % filesize)
+        c.append('%08X' % 0) # major
+        c.append('%08X' % 0) # minor
+        c.append('%08X' % 0) # rmajor
+        c.append('%08X' % 0) # rminor
+        c.append('%08X' % namesize)
+        c.append('%08X' % 0) # checksum
+
+        c.append(name + '\0')
+        c.append('\0' * (len(''.join(c)) % 4))
+
+        c.append(content)
+
+        c = ''.join(c)
+        if len(c) % 4:
+            c += '\0' * (4 - len(c) % 4)
+
+        self.cpio += c
+
+    def add_padding(self):
+        if len(self.cpio) % 512:
+            self.cpio += '\0' * (512 - len(self.cpio) % 512)
+
+    def get(self):
+        self.add('TRAILER!!!', '')
+        self.add_padding()
+        return ''.join(self.cpio)
